package kr.co.vividnext.sodalive.admin.charge

import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import org.springframework.stereotype.Service
import java.time.LocalDate
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class AdminChargeStatusService(val repository: AdminChargeStatusQueryRepository) {
    fun getChargeStatus(startDateStr: String, endDateStr: String): List<GetChargeStatusResponse> {
        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
        val startDate = LocalDate.parse(startDateStr, dateTimeFormatter).atTime(0, 0, 0)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val endDate = LocalDate.parse(endDateStr, dateTimeFormatter).atTime(23, 59, 59)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        var totalChargeAmount = 0
        var totalChargeCount = 0L

        val chargeStatusList = repository.getChargeStatus(startDate, endDate)
            .asSequence()
            .map {
                val chargeAmount = if (it.paymentGateWay == PaymentGateway.PG) {
                    it.pgChargeAmount
                } else {
                    it.appleChargeAmount.toInt()
                }

                val chargeCount = it.chargeCount

                totalChargeAmount += chargeAmount
                totalChargeCount += chargeCount

                GetChargeStatusResponse(
                    date = it.date,
                    chargeAmount = chargeAmount,
                    chargeCount = chargeCount,
                    pg = it.paymentGateWay.name
                )
            }
            .toMutableList()

        chargeStatusList.add(
            0,
            GetChargeStatusResponse(
                date = "합계",
                chargeAmount = totalChargeAmount,
                chargeCount = totalChargeCount,
                pg = ""
            )
        )

        return chargeStatusList.toList()
    }

    fun getChargeStatusDetail(
        startDateStr: String,
        paymentGateway: PaymentGateway
    ): List<GetChargeStatusDetailResponse> {
        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
        val startDate = LocalDate.parse(startDateStr, dateTimeFormatter).atTime(0, 0, 0)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val endDate = LocalDate.parse(startDateStr, dateTimeFormatter).atTime(23, 59, 59)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        return repository.getChargeStatusDetail(startDate, endDate, paymentGateway)
            .asSequence()
            .map {
                GetChargeStatusDetailResponse(
                    memberId = it.memberId,
                    nickname = it.nickname,
                    method = it.method,
                    amount = it.appleChargeAmount.toInt(),
                    datetime = it.datetime
                )
            }
            .toList()
    }
}
