package kr.co.vividnext.sodalive.admin.content.theme

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.DeleteMapping
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@RequestMapping("/admin/audio-content/theme")
@PreAuthorize("hasRole('ADMIN')")
class AdminContentThemeController(private val service: AdminContentThemeService) {
    @PostMapping
    fun enrollmentTheme(
        @RequestPart("image") image: MultipartFile,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.uploadThemeImage(image, requestString), "등록되었습니다.")

    @DeleteMapping("/{id}")
    fun deleteTheme(@PathVariable id: Long) = ApiResponse.ok(service.deleteTheme(id), "삭제되었습니다.")

    @PutMapping("/orders")
    fun updateTagOrders(
        @RequestBody request: UpdateThemeOrdersRequest
    ) = ApiResponse.ok(service.updateTagOrders(request.ids), "수정되었습니다.")

    @GetMapping
    fun getThemes() = ApiResponse.ok(service.getThemes())
}
