package kr.co.vividnext.sodalive.admin.live

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@RequestMapping("/admin/live")
class AdminLiveController(private val service: AdminLiveService) {
    @PostMapping("/cancel-not-started")
    @PreAuthorize("hasRole('BOT')")
    fun cancelNotStartedLive4hoursPassed() = ApiResponse.ok(service.cancelNotStartedLive4hoursPassed())

    @GetMapping
    @PreAuthorize("hasRole('ADMIN')")
    fun getOnAirLive() = ApiResponse.ok(data = service.getLiveList())

    @GetMapping("/recommend-creator")
    @PreAuthorize("hasRole('ADMIN')")
    fun getRecommendCreatorBanner(pageable: Pageable) = ApiResponse.ok(service.getRecommendCreator(pageable))

    @PostMapping("/recommend-creator")
    @PreAuthorize("hasRole('ADMIN')")
    fun createRecommendCreatorBanner(
        @RequestParam("image") image: MultipartFile,
        @RequestParam("creator_id") creatorId: Long,
        @RequestParam("start_date") startDate: String,
        @RequestParam("end_date") endDate: String,
        @RequestParam("is_adult") isAdult: Boolean
    ) = ApiResponse.ok(
        service.createRecommendCreatorBanner(image, creatorId, startDate, endDate, isAdult),
        "등록되었습니다."
    )

    @PutMapping("/recommend-creator")
    @PreAuthorize("hasRole('ADMIN')")
    fun updateRecommendCreatorBanner(
        @RequestParam("recommend_creator_banner_id") recommendCreatorBannerId: Long,
        @RequestParam("image", required = false) image: MultipartFile?,
        @RequestParam("creator_id", required = false) creatorId: Long?,
        @RequestParam("start_date", required = false) startDate: String?,
        @RequestParam("end_date", required = false) endDate: String?,
        @RequestParam("is_adult", required = false) isAdult: Boolean?
    ) = ApiResponse.ok(
        service.updateRecommendCreatorBanner(recommendCreatorBannerId, image, creatorId, startDate, endDate, isAdult),
        "수정되었습니다."
    )

    @PutMapping("/recommend-creator/orders")
    @PreAuthorize("hasRole('ADMIN')")
    fun updateRecommendCreatorBannerOrders(
        @RequestBody request: UpdateAdminRecommendCreatorBannerOrdersRequest
    ) = ApiResponse.ok(
        service.updateRecommendCreatorBannerOrders(request.firstOrders, request.ids),
        "수정되었습니다."
    )
}
