package kr.co.vividnext.sodalive.admin.live.signature

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.signature.QSignatureCan.signatureCan
import kr.co.vividnext.sodalive.live.signature.SignatureCan
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository

interface AdminSignatureCanRepository : JpaRepository<SignatureCan, Long>, AdminSignatureCanQueryRepository

interface AdminSignatureCanQueryRepository {
    fun getSignatureCanListTotalCount(): Int

    fun getSignatureCanList(imageHost: String, offset: Long, limit: Long): List<GetSignatureCanListItem>
}

class AdminSignatureCanQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AdminSignatureCanQueryRepository {
    override fun getSignatureCanListTotalCount(): Int {
        return queryFactory.select(signatureCan.id)
            .from(signatureCan)
            .where(signatureCan.isActive.isTrue)
            .fetch()
            .size
    }

    override fun getSignatureCanList(imageHost: String, offset: Long, limit: Long): List<GetSignatureCanListItem> {
        return queryFactory.select(
            QGetSignatureCanListItem(
                signatureCan.id,
                signatureCan.can,
                signatureCan.image.prepend("/").prepend(imageHost),
                signatureCan.isAdult,
                member.nickname
            )
        )
            .from(signatureCan)
            .innerJoin(signatureCan.creator, member)
            .where(signatureCan.isActive.isTrue)
            .offset(offset)
            .limit(limit)
            .orderBy(signatureCan.id.desc())
            .fetch()
    }
}
