package kr.co.vividnext.sodalive.admin.live.signature

import com.amazonaws.services.s3.model.ObjectMetadata
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.live.signature.SignatureCan
import kr.co.vividnext.sodalive.member.MemberRepository
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.domain.Pageable
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class AdminSignatureCanService(
    private val repository: AdminSignatureCanRepository,
    private val memberRepository: MemberRepository,

    private val s3Uploader: S3Uploader,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String,
    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun getSignatureCanList(pageable: Pageable): GetSignatureCanListResponse {
        val totalCount = repository.getSignatureCanListTotalCount()
        val items = repository.getSignatureCanList(
            imageHost = imageHost,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )

        return GetSignatureCanListResponse(totalCount, items)
    }

    @Transactional
    fun createSignatureCan(can: Int, creatorId: Long, image: MultipartFile, isAdult: Boolean) {
        if (creatorId < 1) throw SodaException("올바른 크리에이터를 선택해 주세요.")

        val creator = memberRepository.findCreatorByIdOrNull(memberId = creatorId)
            ?: throw SodaException("올바른 크리에이터를 선택해 주세요.")

        val signatureCan = SignatureCan(can = can, isAdult = isAdult)
        signatureCan.creator = creator
        repository.save(signatureCan)

        val metadata = ObjectMetadata()
        metadata.contentLength = image.size
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "signature_can/${signatureCan.id}/${generateFileName()}",
            metadata = metadata
        )

        signatureCan.image = imagePath
    }

    @Transactional
    fun modifySignatureCan(id: Long, can: Int?, image: MultipartFile?, isActive: Boolean?, isAdult: Boolean?) {
        val signatureCan = repository.findByIdOrNull(id = id)
            ?: throw SodaException("잘못된 요청입니다.")

        if (can != null) {
            signatureCan.can = can
        }

        if (isActive != null) {
            signatureCan.isActive = isActive
        }

        if (isAdult != null) {
            signatureCan.isAdult = isAdult
        }

        if (image != null) {
            val metadata = ObjectMetadata()
            metadata.contentLength = image.size
            val imagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = bucket,
                filePath = "signature_can/${signatureCan.id}/${generateFileName()}",
                metadata = metadata
            )

            signatureCan.image = imagePath
        }
    }
}
