package kr.co.vividnext.sodalive.content.comment

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
class AudioContentCommentController(private val service: AudioContentCommentService) {
    @PostMapping("/audio-content/comment")
    fun registerComment(
        @RequestBody request: RegisterCommentRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.registerComment(
                comment = request.comment,
                audioContentId = request.contentId,
                parentId = request.parentId,
                member = member
            )
        )
    }

    @PutMapping("/audio-content/comment")
    fun modifyComment(
        @RequestBody request: ModifyCommentRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.modifyComment(request = request, member = member))
    }

    @GetMapping("/audio-content/{id}/comment")
    fun getCommentList(
        @PathVariable("id") audioContentId: Long,
        @RequestParam timezone: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getCommentList(
                audioContentId = audioContentId,
                timezone = timezone,
                pageable = pageable
            )
        )
    }

    @GetMapping("/audio-content/comment/{id}")
    fun getCommentReplyList(
        @PathVariable("id") commentId: Long,
        @RequestParam timezone: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ): ApiResponse<GetAudioContentCommentListResponse> {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        return ApiResponse.ok(
            service.getCommentReplyList(
                commentId = commentId,
                timezone = timezone,
                pageable = pageable
            )
        )
    }
}
