package kr.co.vividnext.sodalive.content.main.curation

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.SortType
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.QGetAudioContentMainItem
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.stereotype.Repository

@Repository
class AudioContentCurationQueryRepository(private val queryFactory: JPAQueryFactory) {
    fun findTotalCountByCurationId(curationId: Long, isAdult: Boolean = false): Int {
        var where = audioContent.isActive.isTrue
            .and(audioContent.curation.id.eq(curationId))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .where(where)
            .fetch()
            .size
    }

    fun findByCurationId(
        curationId: Long,
        cloudfrontHost: String,
        isAdult: Boolean = false,
        sortType: SortType = SortType.NEWEST,
        offset: Long = 0,
        limit: Long = 10
    ): List<GetAudioContentMainItem> {
        val orderBy = when (sortType) {
            SortType.NEWEST -> audioContent.createdAt.desc()
            SortType.PRICE_HIGH -> audioContent.price.desc()
            SortType.PRICE_LOW -> audioContent.price.asc()
        }

        var where = audioContent.isActive.isTrue
            .and(audioContent.member.isNotNull)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.member.isActive.isTrue)
            .and(audioContent.curation.id.eq(curationId))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(cloudfrontHost),
                    audioContent.title,
                    member.id,
                    member.profileImage.prepend("/").prepend(cloudfrontHost),
                    member.nickname,
                    audioContent.price,
                    audioContent.duration
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(orderBy)
            .fetch()
    }
}
