package kr.co.vividnext.sodalive.content.main.curation

import kr.co.vividnext.sodalive.content.SortType
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service

@Service
class AudioContentCurationService(
    private val repository: AudioContentCurationQueryRepository,
    private val blockMemberRepository: BlockMemberRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    fun getCurationContent(
        curationId: Long,
        sortType: SortType,
        member: Member,
        pageable: Pageable
    ): GetCurationContentResponse {
        val totalCount = repository.findTotalCountByCurationId(curationId, member.auth != null)

        val audioContentList = repository.findByCurationId(
            curationId = curationId,
            cloudfrontHost = cloudFrontHost,
            isAdult = member.auth != null,
            sortType = sortType,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )
            .asSequence()
            .filter { !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.creatorId) }
            .toList()

        return GetCurationContentResponse(
            totalCount = totalCount,
            items = audioContentList
        )
    }
}
