package kr.co.vividnext.sodalive.content.theme

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.SortType
import kr.co.vividnext.sodalive.content.theme.content.GetContentByThemeResponse
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AudioContentThemeService(
    private val queryRepository: AudioContentThemeQueryRepository,
    private val blockMemberRepository: BlockMemberRepository,
    private val contentRepository: AudioContentRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    @Transactional(readOnly = true)
    fun getThemes(): List<GetAudioContentThemeResponse> {
        return queryRepository.getActiveThemes()
    }

    @Transactional(readOnly = true)
    fun getContentByTheme(
        themeId: Long,
        sortType: SortType,
        member: Member,
        offset: Long,
        limit: Long
    ): GetContentByThemeResponse {
        val theme = queryRepository.findThemeByIdAndActive(themeId)
            ?: throw SodaException("잘못된 요청입니다.")

        val totalCount = contentRepository.totalCountByTheme(
            memberId = member.id!!,
            theme = theme.theme,
            isAdult = member.auth != null
        )

        val items = contentRepository.findByTheme(
            cloudfrontHost = imageHost,
            memberId = member.id!!,
            theme = theme.theme,
            sortType = sortType,
            isAdult = member.auth != null,
            offset = offset,
            limit = limit
        )
            .asSequence()
            .filter { !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.creatorId) }
            .toList()

        return GetContentByThemeResponse(
            theme = theme.theme,
            totalCount = totalCount,
            items = items
        )
    }
}
