package kr.co.vividnext.sodalive.creator.admin.signature

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@PreAuthorize("hasRole('CREATOR')")
@RequestMapping("/creator-admin/signature")
class CreatorAdminSignatureController(private val service: CreatorAdminSignatureService) {
    @GetMapping
    fun getSignatureCanList(
        pageable: Pageable,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(data = service.getSignatureList(pageable, memberId = member.id!!))
    }

    @PostMapping
    fun createSignature(
        @RequestParam("can") can: Int,
        @RequestParam("image") image: MultipartFile,
        @RequestParam("isAdult", required = false) isAdult: Boolean = false,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.createSignature(can = can, image = image, isAdult = isAdult, memberId = member.id!!),
            "등록되었습니다."
        )
    }

    @PutMapping
    fun modifySignature(
        @RequestParam("id") id: Long,
        @RequestParam("can", required = false) can: Int?,
        @RequestParam("image", required = false) image: MultipartFile?,
        @RequestParam("isActive", required = false) isActive: Boolean?,
        @RequestParam("isAdult", required = false) isAdult: Boolean?,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        if (can == null && image == null && isActive == null && isAdult == null) {
            throw SodaException("변경사항이 없습니다.")
        }

        ApiResponse.ok(
            service.modifySignature(
                id = id,
                can = can,
                image = image,
                isActive = isActive,
                isAdult = isAdult,
                memberId = member.id!!
            ),
            if (isActive == false) "삭제되었습니다." else "수정되었습니다."
        )
    }
}
