package kr.co.vividnext.sodalive.creator.admin.signature

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.signature.QSignatureCan.signatureCan
import kr.co.vividnext.sodalive.live.signature.SignatureCan
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository

interface CreatorAdminSignatureRepository : JpaRepository<SignatureCan, Long>, CreatorAdminSignatureQueryRepository

interface CreatorAdminSignatureQueryRepository {
    fun getSignatureListTotalCount(memberId: Long): Int

    fun getSignatureList(memberId: Long, imageHost: String, offset: Long, limit: Long): List<GetSignatureListItem>

    fun findSignatureByIdOrNull(id: Long, memberId: Long): SignatureCan?
}

class CreatorAdminSignatureQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : CreatorAdminSignatureQueryRepository {
    override fun getSignatureListTotalCount(memberId: Long): Int {
        return queryFactory.select(signatureCan.id)
            .from(signatureCan)
            .where(
                signatureCan.isActive.isTrue
                    .and(signatureCan.creator.id.eq(memberId))
            )
            .fetch()
            .size
    }

    override fun getSignatureList(
        memberId: Long,
        imageHost: String,
        offset: Long,
        limit: Long
    ): List<GetSignatureListItem> {
        return queryFactory.select(
            QGetSignatureListItem(
                signatureCan.id,
                signatureCan.can,
                signatureCan.image.prepend("/").prepend(imageHost),
                signatureCan.isAdult,
                member.nickname
            )
        )
            .from(signatureCan)
            .innerJoin(signatureCan.creator, member)
            .where(
                signatureCan.isActive.isTrue
                    .and(signatureCan.creator.id.eq(memberId))
            )
            .offset(offset)
            .limit(limit)
            .orderBy(signatureCan.id.desc())
            .fetch()
    }

    override fun findSignatureByIdOrNull(id: Long, memberId: Long): SignatureCan? {
        return queryFactory
            .selectFrom(signatureCan)
            .where(
                signatureCan.id.eq(id)
                    .and(signatureCan.creator.id.eq(memberId))
            )
            .fetchFirst()
    }
}
