package kr.co.vividnext.sodalive.explorer.profile.creatorCommunity

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.QCreatorCommunity.creatorCommunity
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.following.QCreatorFollowing.creatorFollowing
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.LocalTime

interface CreatorCommunityRepository : JpaRepository<CreatorCommunity, Long>, CreatorCommunityQueryRepository

interface CreatorCommunityQueryRepository {
    fun findByIdAndMemberId(id: Long, memberId: Long): CreatorCommunity?

    fun getCommunityPostList(
        creatorId: Long,
        offset: Long,
        limit: Long,
        isAdult: Boolean
    ): List<CreatorCommunity>

    fun findByIdAndActive(postId: Long, isAdult: Boolean): CreatorCommunity?
    fun getLatestPostListFromCreatorsYouFollow(memberId: Long, isAdult: Boolean): List<CreatorCommunity>
}

class CreatorCommunityQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : CreatorCommunityQueryRepository {
    override fun findByIdAndMemberId(id: Long, memberId: Long): CreatorCommunity? {
        return queryFactory
            .selectFrom(creatorCommunity)
            .where(
                creatorCommunity.id.eq(id)
                    .and(creatorCommunity.member.id.eq(memberId))
            )
            .fetchFirst()
    }

    override fun getCommunityPostList(
        creatorId: Long,
        offset: Long,
        limit: Long,
        isAdult: Boolean
    ): List<CreatorCommunity> {
        var where = creatorCommunity.member.id.eq(creatorId)
            .and(creatorCommunity.isActive.isTrue)

        if (!isAdult) {
            where = where.and(creatorCommunity.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(creatorCommunity)
            .innerJoin(creatorCommunity.member, member)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(creatorCommunity.createdAt.desc())
            .fetch()
    }

    override fun findByIdAndActive(postId: Long, isAdult: Boolean): CreatorCommunity? {
        var where = creatorCommunity.id.eq(postId)
            .and(creatorCommunity.isActive.isTrue)

        if (!isAdult) {
            where = where.and(creatorCommunity.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(creatorCommunity)
            .where(where)
            .fetchFirst()
    }

    override fun getLatestPostListFromCreatorsYouFollow(memberId: Long, isAdult: Boolean): List<CreatorCommunity> {
        val creatorCommunity = QCreatorCommunity.creatorCommunity
        val latest = QCreatorCommunity.creatorCommunity

        val localDate = LocalDate.now().minusDays(7)
        val localTime = LocalTime.of(0, 0)
        var where = creatorCommunity.isActive.isTrue
            .and(creatorCommunity.createdAt.after(LocalDateTime.of(localDate, localTime)))

        if (!isAdult) {
            where = where.and(creatorCommunity.isAdult.isFalse)
        }

        val subQuery = queryFactory
            .select(latest.member.id, latest.createdAt.max().`as`("maxCreatedAt"))
            .from(latest)
            .where(where)
            .groupBy(latest.member.id)

        where = where.and(
            Expressions.list(creatorCommunity.member.id, creatorCommunity.createdAt).`in`(subQuery)
        )

        val memberSubQuery = queryFactory
            .select(creatorFollowing.creator.id)
            .from(creatorFollowing)
            .where(
                creatorFollowing.member.id.eq(memberId)
                    .and(creatorFollowing.isActive.isTrue)
            )

        where = where.and(
            creatorCommunity.member.id.`in`(memberSubQuery)
        )

        return queryFactory
            .selectFrom(creatorCommunity)
            .where(where)
            .orderBy(creatorCommunity.createdAt.desc())
            .limit(10)
            .fetch()
    }
}
