package kr.co.vividnext.sodalive.live.recommend

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/live/recommend")
class LiveRecommendController(private val service: LiveRecommendService) {
    @GetMapping
    fun getRecommendLive(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getRecommendLive(member))
    }

    @GetMapping("/channel")
    fun getRecommendChannelList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getRecommendChannelList(member))
    }

    @GetMapping("/following/channel/list")
    fun getFollowingChannelList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getFollowingChannelList(member))
    }

    @GetMapping("/following/channel/all/list")
    fun getFollowingAllChannelList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getFollowingAllChannelList(member, pageable))
    }
}
