package kr.co.vividnext.sodalive.member.block

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface BlockMemberRepository : JpaRepository<BlockMember, Long>, BlockMemberQueryRepository

interface BlockMemberQueryRepository {
    fun getBlockAccount(blockedMemberId: Long, memberId: Long): BlockMember?
    fun isBlocked(blockedMemberId: Long, memberId: Long): Boolean
    fun getBlockedMemberList(creatorId: Long): List<Long>
}

@Repository
class BlockMemberQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : BlockMemberQueryRepository {
    override fun getBlockAccount(blockedMemberId: Long, memberId: Long): BlockMember? {
        return queryFactory
            .selectFrom(blockMember)
            .where(
                blockMember.blockedMemberId.eq(blockedMemberId)
                    .and(blockMember.memberId.eq(memberId))
            )
            .orderBy(blockMember.id.desc())
            .fetchFirst()
    }

    override fun isBlocked(blockedMemberId: Long, memberId: Long): Boolean {
        val blockedAccount = queryFactory
            .select(blockMember.id)
            .from(blockMember)
            .where(
                blockMember.memberId.eq(memberId)
                    .and(blockMember.blockedMemberId.eq(blockedMemberId))
                    .and(blockMember.isActive.isTrue)
            )
            .fetchOne()

        return blockedAccount != null
    }

    override fun getBlockedMemberList(creatorId: Long): List<Long> {
        return queryFactory
            .select(blockMember.blockedMemberId)
            .from(blockMember)
            .where(
                blockMember.memberId.eq(creatorId)
                    .and(blockMember.isActive.isTrue)
            )
            .fetch()
    }
}
